from Classification.Instance.Instance cimport Instance
from Corpus.Sentence cimport Sentence
from DataGenerator.InstanceGenerator.InstanceGenerator cimport InstanceGenerator


cdef class DisambiguationInstanceGenerator(InstanceGenerator):

    cpdef addAttributesForPreviousWords(self, Instance current, Sentence sentence, int wordIndex)
    cpdef addAttributesForEmptyWords(self, Instance current, str emptyWord)
    cpdef Instance generateInstanceFromSentence(self, Sentence sentence, int wordIndex)



from Classification.Instance.Instance cimport Instance
from Corpus.Sentence cimport Sentence


cdef class InstanceGenerator:

    cdef int windowSize

    cpdef Instance generateInstanceFromSentence(self, Sentence sentence, int wordIndex)



from AnnotatedSentence.ViewLayerType import ViewLayerType
from AnnotatedTree.TreeBankDrawable cimport TreeBankDrawable
from AnnotatedTree.ParseTreeDrawable cimport ParseTreeDrawable
from NamedEntityRecognition.NERCorpus cimport NERCorpus
from AnnotatedSentence.AnnotatedSentence cimport AnnotatedSentence


cdef class NERCorpusGenerator:

    cdef TreeBankDrawable __treeBank

    def __init__(self, folder: str, pattern: str):
        """
        Constructor for the NERCorpusGenerator which takes input the data directory and the pattern for the
        training files included. The constructor loads the treebank from the given directory including the given files
        the given pattern.

        PARAMETERS
        ----------
        folder : str
            Directory where the treebank files reside.
        pattern : str
            Pattern of the tree files to be included in the treebank. Use "." for all files.
        """
        self.__treeBank = TreeBankDrawable(folder, pattern)

    cpdef NERCorpus generate(self):
        """
        Creates a morphological disambiguation corpus from the treeBank. Calls generateAnnotatedSentence for each parse
        tree in the treebank.

        RETURNS
        -------
        DisambiguationCorpus
            Created disambiguation corpus.
        """
        cdef NERCorpus corpus
        cdef int i
        cdef ParseTreeDrawable parseTree
        cdef AnnotatedSentence sentence
        corpus = NERCorpus()
        for i in range(self.__treeBank.size()):
            parseTree = self.__treeBank.get(i)
            if parseTree.layerAll(ViewLayerType.NER):
                sentence = parseTree.generateAnnotatedSentence()
                corpus.addSentence(sentence)
        return corpus



from Classification.Attribute.BinaryAttribute cimport BinaryAttribute
from Dictionary.Word cimport Word


cdef class IsMoneyAttribute(BinaryAttribute):

    def __init__(self, surfaceForm: str):
        """
        Binary attribute for a given word. If the word is "dolar", "euro", "sterlin", etc., the attribute will have the
        value "true", otherwise "false".

        PARAMETERS
        ----------
        surfaceForm : str
            Surface form of the word.
        """
        super().__init__(Word.isMoney(surfaceForm))



from DataGenerator.DatasetGenerator.DataSetGenerator cimport DataSetGenerator
from DataGenerator.InstanceGenerator.RootWordInstanceGenerator cimport RootWordInstanceGenerator


class RootWordDataSetGenerator(DataSetGenerator):

    def __init__(self, folder: str, pattern: str, rootWordInstanceGenerator: RootWordInstanceGenerator):
        """
        Constructor for the RootWordDataSetGenerator which takes input the data directory, the pattern for the training
        files included, and an instanceGenerator. The constructor calls its super class with these inputs.

        PARAMETERS
        ----------
        folder : str
            Directory where the treebank files reside.
        pattern : str
            Pattern of the tree files to be included in the treebank. Use "." for all files.
        rootWordInstanceGenerator : RootWordInstanceGenerator
            The instance generator used to generate the dataset.
        """
        super().__init__(folder, pattern, rootWordInstanceGenerator)



from Classification.Instance.Instance cimport Instance
from Corpus.Sentence cimport Sentence
from MorphologicalAnalysis.FsmMorphologicalAnalyzer cimport FsmMorphologicalAnalyzer
from WordNet.WordNet cimport WordNet
from DataGenerator.InstanceGenerator.SimpleWindowInstanceGenerator cimport SimpleWindowInstanceGenerator


cdef class SemanticInstanceGenerator(SimpleWindowInstanceGenerator):

    cdef FsmMorphologicalAnalyzer __fsm
    cdef WordNet __wordNet

    cpdef addAttributesForWords(self, Instance current, Sentence sentence, int wordIndex)
    cpdef addAttributesForEmptyWords(self, Instance current, str emptyWord)
    cpdef Instance generateInstanceFromSentence(self, Sentence sentence, int wordIndex)



from Classification.Attribute.BinaryAttribute cimport BinaryAttribute
from MorphologicalAnalysis.MorphologicalParse cimport MorphologicalParse


class IsRealAttribute(BinaryAttribute):

    def __init__(self, parse: MorphologicalParse):
        """
        Binary attribute for a given word. If the word is represents a real number (if the morphological parse contains
        tag REAL), the attribute will have the value "true", otherwise "false".

        PARAMETERS
        ----------
        parse : MorphologicalParse
            Morphological parse of the word.
        """
        super().__init__(parse.isReal())



from AnnotatedSentence.AnnotatedSentence cimport AnnotatedSentence
from AnnotatedSentence.AnnotatedWord cimport AnnotatedWord
from Classification.Instance.CompositeInstance cimport CompositeInstance
from WordNet.SynSet cimport SynSet


cdef class SemanticInstanceGenerator(SimpleWindowInstanceGenerator):

    def __init__(self, fsm: FsmMorphologicalAnalyzer, wordNet: WordNet):
        """
        Constructor for the semantic instance generator. Takes morphological analyzer and wordnet as input to set the
        corresponding variables.

        PARAMETERS
        ----------
        fsm : FsmMorphologicalAnalyzer
            Morphological analyzer to be used.
        wordNet : WordNet
            Wordnet to be used.
        """
        self.__fsm = fsm
        self.__wordNet = wordNet

    cpdef addAttributesForWords(self, Instance current, Sentence sentence, int wordIndex):
        """
        Generates a single classification instance of the WSD problem for the given word of the given sentence. If the
        word has not been labeled with sense tag yet, the method returns null. In the WSD problem, the system also
        generates and stores all possible sense labels for the current instance. In this case, a classification
        instance will not have all labels in the dataset, but some subset of it.

        PARAMETERS
        ----------
        sentence : Sentence
            Input sentence.
        wordIndex : int
            The index of the word in the sentence.

        RETURNS
        -------
        Instance
            Classification instance.
        """
        pass

    cpdef addAttributesForEmptyWords(self, Instance current, str emptyWord):
        pass

    cpdef Instance generateInstanceFromSentence(self, Sentence sentence, int wordIndex):
        cdef list possibleSynSets, possibleClassLabels
        cdef AnnotatedWord word
        cdef str classLabel
        cdef CompositeInstance current
        cdef SynSet synSet
        if isinstance(sentence, AnnotatedSentence):
            possibleSynSets = sentence.constructSynSets(self.__wordNet, self.__fsm, wordIndex)
            word = sentence.getWord(wordIndex)
            if isinstance(word, AnnotatedWord):
                classLabel = word.getSemantic()
                current = CompositeInstance(classLabel)
                possibleClassLabels = []
                for synSet in possibleSynSets:
                    possibleClassLabels.append(synSet.getId())
                current.setPossibleClassLabels(possibleClassLabels)
                self.addAttributes(current, sentence, wordIndex)
                return current



from AnnotatedSentence.AnnotatedWord cimport AnnotatedWord


cdef class RootWordInstanceGenerator(InstanceGenerator):

    cpdef addAttributesForPreviousWords(self, Instance current, Sentence sentence, int wordIndex):
        pass

    cpdef addAttributesForEmptyWords(self, Instance current, str emptyWord):
        pass

    cpdef Instance generateInstanceFromSentence(self, Sentence sentence, int wordIndex):
        """
        Generates a single classification instance of the root word detection problem for the given word of the
        given sentence. If the word does not have a morphological parse, the method throws InstanceNotGenerated.

        PARAMETERS
        ----------
        sentence : Sentence
            Input sentence.
        wordIndex : int
            The index of the word in the sentence.

        RETURNS
        -------
        Instance
            Classification instance.
        """
        cdef AnnotatedWord word
        cdef Instance instance
        cdef int i
        word = sentence.getWord(wordIndex)
        if isinstance(word, AnnotatedWord):
            current = Instance(word.getParse().getWord().getName())
            for i in range(self.windowSize):
                if wordIndex - self.windowSize + i >= 0:
                    self.addAttributesForPreviousWords(current, sentence, wordIndex - self.windowSize + i)
                else:
                    self.addAttributesForEmptyWords(current, "<s>")
            self.addAttributesForPreviousWords(current, sentence, wordIndex)
            return current



from AnnotatedSentence.AnnotatedWord cimport AnnotatedWord
from Classification.Instance.Instance cimport Instance
from Corpus.Sentence cimport Sentence
from DataGenerator.InstanceGenerator.SimpleWindowInstanceGenerator cimport SimpleWindowInstanceGenerator


cdef class ShallowParseInstanceGenerator(SimpleWindowInstanceGenerator):

    cpdef addAttributesForWords(self, Instance current, Sentence sentence, int wordIndex):
        pass

    cpdef addAttributesForEmptyWords(self, Instance current, str emptyWord):
        pass

    cpdef Instance generateInstanceFromSentence(self, Sentence sentence, int wordIndex):
        """
        Generates a single classification instance of the Shallow Parse problem for the given word of the given
        sentence. If the  word has not been labeled with shallow parse tag yet, the method returns null.

        PARAMETERS
        ----------
        sentence : Sentence
            Input sentence.
        wordIndex : int
            The index of the word in the sentence.

        RETURNS
        -------
        Instance
            Classification instance.
        """
        cdef AnnotatedWord word
        cdef str classLabel
        cdef Instance current
        word = sentence.getWord(wordIndex)
        if isinstance(word, AnnotatedWord):
            classLabel = word.getShallowParse()
            current = Instance(classLabel)
            self.addAttributes(current, sentence, wordIndex)
            return current



from AnnotatedSentence.AnnotatedCorpus cimport AnnotatedCorpus
from AnnotatedSentence.AnnotatedSentence cimport AnnotatedSentence
from Classification.DataSet.DataSet cimport DataSet
from DataGenerator.InstanceGenerator.InstanceGenerator cimport InstanceGenerator


cdef class AnnotatedDatasetGenerator:

    cdef AnnotatedCorpus __corpus
    cdef InstanceGenerator instanceGenerator

    def __init__(self, folder: str, pattern: str, instanceGenerator: InstanceGenerator):
        """
        Constructor for the AnnotatedDataSetGenerator which takes input the data directory, the pattern for the
        training files included, and an instanceGenerator. The constructor loads the sentence corpus from the given
        directory including the given files having the given pattern.

        PARAMETERS
        ----------
        folder : str
            Directory where the corpus files reside.
        pattern : str
            Pattern of the tree files to be included in the treebank. Use "." for all files.
        instanceGenerator : InstanceGenerator
            The instance generator used to generate the dataset.
        """
        self.__corpus = AnnotatedCorpus(folder, pattern)
        self.instanceGenerator = instanceGenerator

    cpdef setInstanceGenerator(self, InstanceGenerator instanceGenerator):
        """
        Mutator for the instanceGenerator attribute.

        PARAMETERS
        ----------
        instanceGenerator : InstanceGenerator
            Input instanceGenerator
        """
        self.instanceGenerator = instanceGenerator

    cpdef DataSet generate(self):
        """
        Creates a dataset from the corpus. Calls generateInstanceFromSentence for each parse sentence in the corpus.

        RETURNS
        -------
        DataSet
            Created dataset.
        """
        cdef DataSet dataSet
        cdef AnnotatedSentence sentence, generatedInstance
        cdef int j
        dataSet = DataSet()
        for sentence in self.__corpus.sentences:
            for j in range(sentence.wordCount()):
                generatedInstance = self.instanceGenerator.generateInstanceFromSentence(sentence, j)
                if generatedInstance is not None:
                    dataSet.addInstance(generatedInstance)
        return dataSet



from NamedEntityRecognition.NamedEntityType import NamedEntityType
from AnnotatedSentence.AnnotatedWord cimport AnnotatedWord


cdef class NerInstanceGenerator(SimpleWindowInstanceGenerator):

    cpdef addAttributesForWords(self, Instance current, Sentence sentence, int wordIndex):
        pass

    cpdef addAttributesForEmptyWords(self, Instance current, str emptyWord):
        pass

    cpdef Instance generateInstanceFromSentence(self, Sentence sentence, int wordIndex):
        cdef AnnotatedWord word
        cdef str classLabel
        cdef Instance current
        word = sentence.getWord(wordIndex)
        if isinstance(word, AnnotatedWord):
            classLabel = NamedEntityType.getNamedEntityString(word.getNamedEntityType())
            current = Instance(classLabel)
            self.addAttributes(current, sentence, wordIndex)
            return current



from AnnotatedSentence.AnnotatedSentence cimport AnnotatedSentence
from AnnotatedSentence.AnnotatedWord cimport AnnotatedWord
from AnnotatedSentence.AnnotatedCorpus cimport AnnotatedCorpus
from DisambiguationCorpus.DisambiguatedWord cimport DisambiguatedWord
from DisambiguationCorpus.DisambiguationCorpus cimport DisambiguationCorpus

cdef class SentenceDisambiguationCorpusGenerator:

    cdef AnnotatedCorpus __annotatedCorpus

    def __init__(self, folder: str, pattern: str):
        """
        Constructor for the DisambiguationCorpusGenerator which takes input the data directory and the pattern for the
        training files included. The constructor loads the corpus from the given directory including the given files
        the given pattern.

        PARAMETERS
        ----------
        folder : str
            Directory where the sentence files reside.
        pattern : str
            Pattern of the tree files to be included in the corpus. Use "." for all files.
        """
        self.__annotatedCorpus = AnnotatedCorpus(folder, pattern)

    cpdef DisambiguationCorpus generate(self):
        """
        Creates a morphological disambiguation corpus from the corpus.

        RETURNS
        -------
        DisambiguationCorpus
            Created disambiguation corpus.
        """
        cdef DisambiguationCorpus corpus
        cdef int i
        cdef AnnotatedSentence disambiguationSentence
        cdef AnnotatedWord annotatedWord
        corpus = DisambiguationCorpus()
        for i in range(self.__annotatedCorpus.sentenceCount()):
            sentence = self.__annotatedCorpus.getSentence(i)
            disambiguationSentence = AnnotatedSentence()
            for j in range(sentence.wordCount()):
                annotatedWord = sentence.getWord(j)
                if isinstance(annotatedWord, AnnotatedWord):
                    disambiguationSentence.addWord(DisambiguatedWord(annotatedWord.getName(),
                                                                     annotatedWord.getParse()))
            corpus.addSentence(disambiguationSentence)
        return corpus



from setuptools import setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding="utf-8")
from Cython.Build import cythonize

setup(
    ext_modules=cythonize(["DataGenerator/Attribute/*.pyx",
                           "DataGenerator/CorpusGenerator/*.pyx",
                           "DataGenerator/InstanceGenerator/*.pyx",
                           "DataGenerator/DatasetGenerator/*.pyx"],
                          compiler_directives={'language_level': "3"}),
    name='NlpToolkit-DataGenerator-Cy',
    version='1.0.2',
    packages=['DataGenerator', 'DataGenerator.Attribute', 'DataGenerator.CorpusGenerator',
              'DataGenerator.DatasetGenerator', 'DataGenerator.InstanceGenerator'],
    package_data={'DataGenerator.Attribute': ['*.pxd', '*.pyx', '*.c'],
                  'DataGenerator.CorpusGenerator': ['*.pxd', '*.pyx', '*.c'],
                  'DataGenerator.DatasetGenerator': ['*.pxd', '*.pyx', '*.c'],
                  'DataGenerator.InstanceGenerator': ['*.pxd', '*.pyx', '*.c']},
    url='https://github.com/StarlangSoftware/DataGenerator-Cy',
    license='',
    author='olcaytaner',
    author_email='olcay.yildiz@ozyegin.edu.tr',
    description='Classification dataset generator library for high level Nlp tasks',
    install_requires=['NlpToolkit-AnnotatedTree-Cy', 'NlpToolkit-Classification-Cy', 'NlpToolkit-MorphologicalDisambiguation-Cy'],
    long_description=long_description,
    long_description_content_type='text/markdown'
)



from AnnotatedTree.ParseTreeDrawable cimport ParseTreeDrawable
from AnnotatedTree.TreeBankDrawable cimport TreeBankDrawable
from Classification.DataSet.DataSet cimport DataSet
from DataGenerator.InstanceGenerator.InstanceGenerator cimport InstanceGenerator


cdef class DataSetGenerator:

    cdef TreeBankDrawable __treeBank
    cdef InstanceGenerator instanceGenerator

    cpdef setInstanceGenerator(self, InstanceGenerator instanceGenerator)
    cpdef list generateInstanceListFromTree(self, ParseTreeDrawable parseTree)
    cpdef DataSet generate(self)
